#include <iostream>
#include "wx/wx.h"
#include "wx/print.h"
#include "wx/printdlg.h"
#include <cmath>
#include "wx/stdpaths.h"
#include "wx/dialog.h"

class MyFrame;
MyFrame* frame;


class PrintImage : public wxPrintout
{
    wxBitmap bitmap;
    wxPageSetupDialogData page_setup;
    int orient;
public:
    PrintImage() : wxPrintout( wxT("GIMP Image") )
    {
        orient = wxPORTRAIT;
    }
    void loadPNG(wxString filename)
    {
        bitmap.LoadFile( filename , wxBITMAP_TYPE_PNG);
        if(!bitmap.IsOk()){ std::cerr << "can't load PNG image" << std::endl; exit(1); }
    }
    void loadJPEG(wxString filename)
    {
        bitmap.LoadFile( filename , wxBITMAP_TYPE_JPEG);
        if(!bitmap.IsOk()){ std::cerr << "can't load JPG image" << std::endl; exit(1); }
    }
    bool OnPrintPage(int pageNum)
    {
        wxDC* dc = GetDC();
        
        
        if(orient != wxPORTRAIT)
        {
            // rotate image
            wxImage imageToRotate = bitmap.ConvertToImage();
            wxImage rotatedImage = imageToRotate.Rotate90();
            wxBitmap rotatedBitmap(rotatedImage);
            bitmap = rotatedBitmap;
        }
        
        // center image
        double xscale = 1, yscale = 1;
        dc->GetUserScale( &xscale, &yscale );
        
#define CM_2_INCH(X) (X/2.54)
#define INCH_2_CM(X) (X*2.54)
        
        // 1 in = 2.54 cm
        std::cout << "dimensions : " << (dc->GetSizeMM().GetWidth()/10.0f)*xscale << " cm X " << (dc->GetSizeMM().GetHeight()/10.0f)*yscale << "cm\n";
        std::cout << "dimensions : " <<
            CM_2_INCH((dc->GetSizeMM().GetWidth()/10.0f)*xscale) << " inches X " <<
            CM_2_INCH((dc->GetSizeMM().GetHeight()/10.0f)*yscale) << " inches\n";
        
        const float inch_width = CM_2_INCH((dc->GetSizeMM().GetWidth()/10.0f)*xscale);
        const float inch_height = CM_2_INCH((dc->GetSizeMM().GetHeight()/10.0f)*yscale);
        
        // total_inch_width   --> dc_pixel_width
        // desired_inch_width --> [desired_pixel_width]
        
        // [xscale] = desired_pixel_width/bitmap_pixel_width

        // center image
        int x=0, y=0;
        if(bitmap.GetWidth()*xscale < dc->GetSize().GetWidth())
        {
            x = (int)(dc->GetSize().GetWidth() / 2.0 - bitmap.GetWidth()*xscale/2.0) / xscale;
            if(x<0) x=0;
        }
        
        if(bitmap.GetHeight()*yscale < dc->GetSize().GetHeight())
        {
            y = (int)(dc->GetSize().GetHeight() / 2.0 - bitmap.GetHeight()*yscale/2.0) / yscale;
            if(y<0) y=0;
        }
        
        // draw image
        dc->DrawBitmap(bitmap, x, y, false);
        return true;
    }  
    void preparePrint()
    {
        // show first print dialog (with orientation)
        wxPageSetupDialog dialog( (wxWindow*)frame);
        if(dialog.ShowModal()==wxID_OK)
        {
            page_setup = dialog.GetPageSetupData();
            orient = page_setup.GetPrintData().GetOrientation();
        }else{ std::cout << "user canceled at first dialog" << std::endl; exit(1); }
    }
    void OnBeginPrinting()
    {
        //maximize image
        if(orient == wxPORTRAIT)
            FitThisSizeToPageMargins(wxSize(bitmap.GetWidth(), bitmap.GetHeight()), page_setup);
        else 
            FitThisSizeToPageMargins(wxSize(bitmap.GetHeight(), bitmap.GetWidth()), page_setup);
    }
    void OnEndPrinting()
    {
        exit(0);
    }
};

class MyApp: public wxApp
{
public:
    virtual bool OnInit();
};


IMPLEMENT_APP(MyApp)


bool MyApp::OnInit()
{
    wxInitAllImageHandlers();
    
    wxString imagePath;
    for(int n=0; n<argc; n++)
    {
        if(wxString(argv[n]).Find(wxT(".png")) != wxNOT_FOUND)
        {
            imagePath = wxString(argv[n]);
        }
    }
    
    std::cout << "image : " << imagePath.mb_str() << std::endl;
    
    if(imagePath.IsEmpty())
    {
        wxMessageBox(wxT("An error occurred:\ncould not localize image to print. Aborting."));
        exit(1);
    }
    
    PrintImage myprint;
    
    myprint.loadPNG( imagePath ); 
    myprint.preparePrint();
    
    wxPrinter printer;
    const bool success = printer.Print(NULL, &myprint, true /* show dialog */);

    if(success) exit(0);
    else exit(1);
    
    return true;
}